@@ -4,7 +4,7 @@ mod tokenizer; // Takes the command template that is provided and reduces it to
 mod parser;    // Collects the input arguments given to the program.
 
 use std::io::{self, Write, BufRead};
-use std::process::{Command, exit};
+use std::process::exit;
 
 use std::thread::{self, JoinHandle};
 use std::sync::Arc;
@@ -20,6 +20,9 @@ use parser::{Args, ParseErr};
  - paralllel command ::: a b c :::+ 1 2 3 ::: d e f :::+ 4 5 6
 */
 
+/// The `JobOutput` structure is utilized when grouping is enabled to transmit a command's
+/// associated job ID with it's stdout and stderr buffers back to the main thread to be
+/// queued for printing in the order that the inputs are supplied.
 struct JobOutput {
     id: usize,
     stdout: Vec<u8>,
@@ -36,9 +39,7 @@ fn main() {
         ncores: num_cpus::get(),
         // Defines whether stdout/stderr buffers should be printed in order.
         grouped: true,
-        // Stores the command that will be executed
-        command: String::new(),
-        // Stores a Vec<Token> of the arguments to execute with the command.
+        // Stores a Vec<Token> of the command arguments.
         arguments: Vec::new(),
         // Stores the list of inputs supplied to the program.
         inputs: Vec::new()
@@ -71,8 +72,8 @@ fn main() {
         }
     }
 
-    // If no command was given, then the inputs are actually commands themselves.
-    let input_is_command = args.command.is_empty();
+    // If no command argument was given, then the inputs are actually commands themselves.
+    let input_is_command = args.arguments.is_empty();
 
     // It will be useful to know the number of inputs, to know when to quit.
     let num_inputs = args.inputs.len();
@@ -91,11 +92,11 @@ fn main() {
     // The `threads` vector will contain the thread handles needed to
     // know when to quit the program.
     let mut threads: Vec<JoinHandle<()>> = Vec::with_capacity(args.ncores);
+
+    // The `slot` variable is required by the {%} token.
     for slot in 1..args.ncores+1 {
-        // The command that each input variable will be sent to.
-        let command = args.command.clone();
         // The base command template that each thread will use.
-        let argument_tokens = args.arguments.clone();
+        let arguments = args.arguments.clone();
         // Allow the thread to gain access to the list of inputs.
         let input = shared_input.clone();
         // Allow the thread to access the input counter
@@ -107,7 +108,6 @@ fn main() {
         // Each thread will receive it's own sender for sending stderr/stdout buffers.
         let output_tx = output_tx.clone();
 
-
         // The actual thread where the work will happen on incoming data.
         let handle: JoinHandle<()> = thread::spawn(move || {
             // Grab a handle to standard error for this thread.
@@ -135,22 +135,20 @@ fn main() {
                 };
 
                 if input_is_command {
-                    // The inputs are actually the commands.
-                    let mut iterator = input_var.split_whitespace();
-                    // There will always be at least one argument: the command.
-                    let actual_command = iterator.next().unwrap();
-                    // The rest of the fields are arguments, if there are any arguments.
-                    let args = iterator.collect::<Vec<&str>>();
-                    // Attempt to run the current input as a command.
                     if grouped {
-                        match Command::new(actual_command).args(&args).output() {
+                        // Executes each input as if it were a command and returns a
+                        // `Command::Output`, if the command executes successfully.
+                        match command::get_command_output(input_var) {
                             Ok(ref output) => {
+                                // Assign the `job_id` with the command's stdout and stderr
+                                // buffers and transmit them back to the main thread.
                                 output_tx.send(JobOutput{
                                     id:     job_id,
                                     stdout: output.stdout.clone(),
                                     stderr: output.stderr.clone(),
                                 }).unwrap();
                             },
+                            // The command has, sadly, failed. This will tell the user why.
                             Err(why) => {
                                 let mut stderr = stderr.lock();
                                 let _ = write!(&mut stderr, "parallel: command error: {}: {}\n",
@@ -158,18 +156,22 @@ fn main() {
                             }
                         }
                     } else {
-                        if let Err(why) = Command::new(actual_command).args(&args).status() {
+                        // With no need to group the outputs, we only need to know the status
+                        // of the command's execution. The standard output and standard error
+                        // will automatically be inherited.
+                        if let Err(why) = command::get_command_status(input_var) {
                             let mut stderr = stderr.lock();
-                            let _ = write!(&mut stderr, "parallel: command error: {}: {}\n",
-                                input_var, why);
+                            let _ = stderr.write(b"parallel: command error:");
+                            let _ = write!(&mut stderr, "{}: {}\n", input_var, why);
                         }
                     }
                 } else {
                     // Build a command by merging the command template with the input,
                     // and then execute that command.
-                    match command::exec(input_var, &command, &argument_tokens, &slot,
+                    match command::exec(input_var, &arguments, &slot,
                         &job_id.to_string(), &job_total, grouped)
                     {
+                        // If grouping is enabled, then we have an output to process.
                         Ok(Some(ref output)) if grouped => {
                             output_tx.send(JobOutput{
                                 id: job_id,
@@ -177,7 +179,9 @@ fn main() {
                                 stderr: output.stderr.clone(),
                             }).unwrap();
                         },
+                        // If grouping was not enabled, nothing was returned.
                         Ok(_) => (),
+                        // I've an error handler already created for this error type.
                         Err(cmd_err) => {
                             let mut stderr = stderr.lock();
                             cmd_err.handle(&mut stderr);
@@ -196,8 +200,14 @@ fn main() {
         let stdout = io::stdout();
         let mut stdout = stdout.lock();
         let mut stderr = stderr.lock();
+        // Job ID's start counting from `1`. This counter will keep track of what ID
+        // we need to print next.
         let mut counter = 1;
+        // If the job we receive is ahead of the current counter, we will queue it for
+        // later printing in this `buffer` variable.
         let mut buffer = Vec::new();
+
+        // The loop will only quit once all inputs have been received. I guarantee it.
         while counter != num_inputs + 1 {
             // Block and wait until a new buffer is received.
             let output = input_rx.recv().unwrap();
@@ -214,10 +224,14 @@ fn main() {
             // Check to see if there are any stored buffers that can now be printed.
             // Items in the buffer will be removed after they are used.
             'outer: loop {
+                // Keep track of if any changes have been made in this iteration.
                 let mut changed = false;
+                // Store a list of indexes we need to drop after a match has been found.
                 let mut drop = Vec::new();
 
                 // Loop through the list of buffers and print buffers with the next ID in line.
+                // If a match was found, changed will be set to true and the job added to the
+                // drop list. If no change was found, the outer loop will quit.
                 for (id, output) in buffer.iter().enumerate() {
                     if output.id == counter {
                         let _ = stdout.write(&output.stdout);
@@ -230,6 +244,9 @@ fn main() {
 
                 // Drop the buffers that were used.
                 if !drop.is_empty() {
+                    // Values have to be dropped in reverse because each time a value is
+                    // removed from a vector, all of them items to the right are shifted to
+                    // to he left.
                     drop.sort();
                     for id in drop.iter().rev() {
                         let _ = buffer.remove(*id);
